@@ -34,7 +34,9 @@
 #include <sys/stat.h>
 #include <sys/wait.h>
 #include <sys/ipc.h>
+#ifndef FIO_NO_HAVE_SHM_H
 #include <sys/shm.h>
+#endif
 #include <sys/mman.h>
 
 #include "fio.h"
@@ -801,6 +803,10 @@ static void cleanup_io_u(struct thread_data *td)
 		io_u = flist_entry(entry, struct io_u, list);
 
 		flist_del(&io_u->list);
+
+		if (td->io_ops->io_u_free)
+			td->io_ops->io_u_free(td, io_u);
+
 		fio_memfree(io_u, sizeof(*io_u));
 	}
 
@@ -883,6 +889,16 @@ static int init_io_u(struct thread_data *td)
 		io_u->index = i;
 		io_u->flags = IO_U_F_FREE;
 		flist_add(&io_u->list, &td->io_u_freelist);
+
+		if (td->io_ops->io_u_init) {
+			int ret = td->io_ops->io_u_init(td, io_u);
+
+			if (ret) {
+				log_err("fio: failed to init engine data: %d\n", ret);
+				return 1;
+			}
+		}
+
 		p += max_bs;
 	}
 
@@ -995,6 +1011,8 @@ static void *thread_main(void *data)
 	} else
 		td->pid = gettid();
 
+	fio_local_clock_init(td->o.use_thread);
+
 	dprint(FD_PROCESS, "jobs pid=%d started\n", (int) td->pid);
 
 	INIT_FLIST_HEAD(&td->io_u_freelist);
@@ -1271,8 +1289,8 @@ err:
 		verify_async_exit(td);
 
 	close_and_free_files(td);
-	close_ioengine(td);
 	cleanup_io_u(td);
+	close_ioengine(td);
 	cgroup_shutdown(td, &cgroup_mnt);
 
 	if (td->o.cpumask_set) {
@@ -1383,7 +1401,7 @@ static void reap_threads(unsigned int *nr_running, unsigned int *t_rate,
 			if (WIFSIGNALED(status)) {
 				int sig = WTERMSIG(status);
 
-				if (sig != SIGTERM)
+				if (sig != SIGTERM && sig != SIGUSR2)
 					log_err("fio: pid=%d, got signal=%d\n",
 							(int) td->pid, sig);
 				td->sig = sig;
