@@ -204,7 +204,7 @@ static int fio_splice_write(struct thread_data *td, struct io_u *io_u)
 static int fio_spliceio_queue(struct thread_data *td, struct io_u *io_u)
 {
 	struct spliceio_data *sd = td->io_ops->data;
-	int uninitialized_var(ret);
+	int ret = 0;
 
 	fio_ro_check(td, io_u);
 
@@ -284,7 +284,7 @@ static int fio_spliceio_init(struct thread_data *td)
 	 * buffers. Just set ->odirect to force that.
 	 */
 	if (td_read(td))
-		td->o.odirect = 1;
+		td->o.mem_align = 1;
 
 	td->io_ops->data = sd;
 	return 0;
@@ -311,7 +311,7 @@ static struct ioengine_ops ioengine = {
  */
 static int fio_spliceio_init(struct thread_data fio_unused *td)
 {
-	fprintf(stderr, "fio: splice not available\n");
+	log_err("fio: splice not available\n");
 	return 1;
 }
 
